
from importlib import import_module

from django.apps import apps
from django.contrib import admin
from django.utils.translation import gettext_lazy as _

from ordered_model.admin import OrderedModelAdmin

from attributes.admin.forms import AttributeForm, AttributeOptionInline
from attributes.models import Attribute

from attributes.actions import categories_change_action


def _get_attribute_admin_base_class():

    if apps.is_installed('modeltranslation'):
        return import_module('modeltranslation.admin').TranslationAdmin

    return admin.ModelAdmin


@admin.register(Attribute)
class AttributeAdmin(OrderedModelAdmin, _get_attribute_admin_base_class()):

    form = AttributeForm
    inlines = [AttributeOptionInline]

    list_display = [
        'name', 'get_category_list', 'slug', 'get_type', 'is_required',
        'is_visible', 'is_filter', 'move_up_down_links']
    search_fields = ['name', 'slug']
    list_filter = ['categories', 'type', 'is_required']
    filter_horizontal = ['categories']
    fields = [
        'categories',
        'name',
        ('is_required', 'is_visible', 'is_filter', ),
        ('type', 'slug', ),
    ]

    actions = [categories_change_action]

    def get_category_list(self, item, count=4):
        categories = list(item.categories.all())

        result = ', '.join([c.name for c in categories[:count]])

        if len(categories) > count:
            result += ' (+{})'.format(len(categories) - count)

        return result

    get_category_list.short_description = _('Categories')

    def get_type(self, item):
        return item.get_type_display()

    get_type.short_description = _('Type')




from attributes.models import AttributeValue


def format_attributes(category, entry_ids):

    attrs = []

    attributes = category.attributes.visible()

    values = {attr.id: {} for attr in attributes}

    attr_values_qs = AttributeValue.objects.filter(
        attr__in=attributes,
        entry__in=entry_ids)

    for attr_val in attr_values_qs:
        values[attr_val.attr_id][attr_val.entry_id] = attr_val.as_html()

    for attr in attributes:
        attrs.append({
            'name': attr.name,
            'values': [values[attr.id].get(entry_id) for entry_id in entry_ids]
        })

    return attrs




from django.db import models
from django.conf import settings
from django.forms import CharField, ModelChoiceField
from django.utils.translation import gettext, gettext_lazy as _

from ordered_model.models import (
    OrderedModel,
    OrderedModelManager,
    OrderedModelQuerySet)

from attributes.models.attribute_value import AttributeValue, VALUE_FIELDS
from attributes.constants import ATTR_TYPE_SELECT, ATTR_TYPES


class AttributeQuerySet(OrderedModelQuerySet):

    def visible(self):
        return self.filter(is_visible=True)

    def for_filter(self):
        return self.filter(type=ATTR_TYPE_SELECT, is_filter=True)

    def for_categories(self, categories):
        return self.filter(categories__in=categories)


class AttributeManager(OrderedModelManager):

    def get_queryset(self):
        return AttributeQuerySet(self.model, using=self._db)

    def visible(self):
        return self.get_queryset().visible()

    def for_filter(self):
        return self.get_queryset().for_filter()

    def for_categories(self, categories):
        return self.get_queryset().for_categories(categories)


class Attribute(OrderedModel):

    categories = models.ManyToManyField(
        'categories.Category',
        related_name='attributes',
        blank=True,
        verbose_name=_("Categories"))

    name = models.CharField(
        _('Name'),
        max_length=128)

    slug = models.CharField(
        _('Code'),
        max_length=255,
        db_index=True,
        blank=True,
        null=False)

    type = models.PositiveSmallIntegerField(
        choices=ATTR_TYPES,
        default=ATTR_TYPE_SELECT,
        null=False,
        verbose_name=_("Type"))

    is_required = models.BooleanField(
        _('Required'),
        default=False,
        help_text=_('You will not be able to update record without filling '
                    'this field'))

    is_visible = models.BooleanField(
        _('Is visible'),
        default=True,
        help_text=_('Display this attribute for users'))

    is_filter = models.BooleanField(
        _('Is filter'),
        default=False,
        help_text=_('Display this attribute in records filter'))

    objects = AttributeManager()

    @property
    def has_options(self):
        return self.type == ATTR_TYPE_SELECT

    @property
    def full_slug(self):
        return 'attr_' + self.slug

    @property
    def category_ids_str(self):
        ids = self.categories.all().values_list('id', flat=True)
        return ','.join(map(str, ids))

    def __str__(self):
        return self.name

    def get_form_field_label(self):

        label = self.name

        if self.is_required:
            label += ' *'

        return label

    def get_option_form_field_name(self):
        return 'option_' + self.full_slug

    def build_form_fields(self):

        fields = {self.full_slug: self.build_form_field()}

        if self.has_options:
            fields[self.get_option_form_field_name()] = (
                self.build_option_form_field())

        for field in fields.values():
            field.widget.attrs['data-category-ids'] = self.category_ids_str

        return fields

    def build_form_field(self):

        params = {'label': self.get_form_field_label(), 'required': False}

        if self.type is ATTR_TYPE_SELECT:
            params['queryset'] = self.options.all()
            field =  ModelChoiceField(**params)
        else:
            field = VALUE_FIELDS[self.type].formfield(**params)

        return field

    def build_option_form_field(self):
        label = self.name + gettext(' [New value]')
        return CharField(label=label, required=False)

    def save_value(self, entry, value):

        params = {'entry': entry, 'attr': self}

        if not value:
            AttributeValue.objects.filter(**params).delete()
            return None

        try:
            attr_value = AttributeValue.objects.get(**params)
        except AttributeValue.DoesNotExist:
            attr_value = AttributeValue(**params)

        attr_value.value = value
        attr_value.save()

        return attr_value

    class Meta:
        ordering = ['order']
        verbose_name = _('Attribute')
        verbose_name_plural = _('Attributes')



# Generated by Django 3.0.11 on 2020-12-07 15:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('attributes', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='attribute',
            name='name_ru',
            field=models.CharField(max_length=128, null=True, verbose_name='Name'),
        ),
        migrations.AddField(
            model_name='attribute',
            name='name_uk',
            field=models.CharField(max_length=128, null=True, verbose_name='Name'),
        ),
        migrations.AddField(
            model_name='attributeoption',
            name='name_ru',
            field=models.CharField(max_length=255, null=True, verbose_name='Name'),
        ),
        migrations.AddField(
            model_name='attributeoption',
            name='name_uk',
            field=models.CharField(max_length=255, null=True, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='type',
            field=models.PositiveSmallIntegerField(choices=[(1, 'Text'), (2, 'Integer'), (3, 'True / False'), (5, 'Options')], default=5, verbose_name='Type'),
        ),
    ]




from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


def setup_settings(settings, **kwargs):

    settings['INSTALLED_APPS'] += [
        app for app in [
            'ordered_model'
        ] if app not in settings['INSTALLED_APPS']
    ]


class AttributesAppConfig(AppConfig):
    name = 'attributes'
    verbose_name = _('Attributes')

default_app_config = 'attributes.AttributesAppConfig'




from django import forms
from django.contrib import admin
from django.shortcuts import render, redirect
from django.utils.translation import gettext_lazy as _
from django.contrib import messages

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Submit, Hidden

from categories.models import Category


class ChangeCategoriesForm(forms.Form):

    _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)

    value = forms.ModelChoiceField(
        label=Category._meta.verbose_name,
        queryset=Category.objects.all())

    should_data_be_replaced = forms.BooleanField(
        label=_('Replace existing categories'),
        initial=False,
        required=False
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.add_input(Submit('apply', _('Apply')))
        self.helper.add_input(Hidden('action', 'categories_change_action'))


def categories_change_action(modeladmin, request, queryset):

    data = request.POST

    apply = 'apply' in data

    initial = {
        '_selected_action': data.getlist(admin.ACTION_CHECKBOX_NAME)
    }

    form = ChangeCategoriesForm(
        data=data if apply else None,
        initial=initial if not apply else None)

    if apply and form.is_valid():

        category = form.cleaned_data['value']

        for instance in queryset:
            if form.cleaned_data['should_data_be_replaced']:
                instance.categories.set([category])
            else:
                instance.categories.add(category)

        messages.success(request, _('Changes saved'))

        return redirect(request.get_full_path())

    context = admin.site.each_context(request)

    context.update({
        'items': queryset,
        'form': form
    })

    return render(
        request,
        'change-attr-category-action.html',
        context)


categories_change_action.short_description = _('Change categories')




import six

from django.db import models
from django.utils.translation import gettext_lazy as _

from attributes.managers import AttributeValueManager
from attributes.constants import (
    ATTR_TYPE_TEXT,
    ATTR_TYPE_INT,
    ATTR_TYPE_BOOL,
    ATTR_TYPE_SELECT)


ATTR_VALUE_TEXT = models.TextField(blank=True, null=True)

ATTR_VALUE_INT = models.IntegerField(blank=True, null=True)

ATTR_VALUE_BOOL = models.NullBooleanField(blank=True)

ATTR_VALUE_OPTION = models.ForeignKey(
    'attributes.AttributeOption',
    blank=True,
    null=True,
    related_name='attr_values',
    on_delete=models.SET_NULL)

VALUE_FIELDS = {
    ATTR_TYPE_TEXT: ATTR_VALUE_TEXT,
    ATTR_TYPE_INT: ATTR_VALUE_INT,
    ATTR_TYPE_BOOL: ATTR_VALUE_BOOL,
    ATTR_TYPE_SELECT: ATTR_VALUE_OPTION,
}


class AttributeValue(models.Model):

    attr = models.ForeignKey(
        'attributes.Attribute',
        related_name='values',
        on_delete=models.CASCADE)

    entry = models.ForeignKey(
        'products.Product',
        related_name='attr_values',
        on_delete=models.CASCADE)

    objects = AttributeValueManager()

    value_text = ATTR_VALUE_TEXT
    value_int = ATTR_VALUE_INT
    value_bool = ATTR_VALUE_BOOL
    value_option = ATTR_VALUE_OPTION

    @staticmethod
    def get_value_field(field_type):
        return VALUE_FIELDS[field_type]

    @property
    def value_field(self):
        return self.get_value_field(self.attr.type)

    def get_value(self):
        return getattr(self, self.value_field.name)

    def set_value(self, new_value):

        if self.attr.has_options and isinstance(new_value, six.string_types):

            new_value = self.attr.options.get(option=new_value)

        setattr(self, self.value_field.name, new_value)

    value = property(get_value, set_value)

    def as_text(self):

        value = self.get_value()

        if self.attr.type == ATTR_TYPE_BOOL:

            if value is None:
                return ''

            return _('Yes') if value else _('No')

        return str('' if value is None else value)

    def as_html(self):
        return self.as_text()

    def __str__(self):
        return '{}: {}'.format(self.attr.name, self.as_text())

    class Meta:
        ordering = ('attr__order', )
        unique_together = ('attr', 'entry')




from django import forms
from django.utils.translation import gettext_lazy as _

from attributes.widgets import AttributesFormFieldWidget
from attributes.forms import AttributesForm


class AttributesFormField(forms.Field):

    widget = AttributesFormFieldWidget
    form = None

    def init_form(self, *args, **kwargs):
        self.form = AttributesForm(*args, **kwargs)
        self.widget.form = self.form

    def clean(self, *args, **kwargs):
        if not self.form.is_valid():
            raise forms.ValidationError(_('Form is invalid.'))

        return self.form.cleaned_data

    def commit(self, *args, **kwargs):
        return self.form.commit(*args, **kwargs)




from django.db import models
from django.utils.translation import gettext_lazy as _


class AttributeOption(models.Model):

    attr = models.ForeignKey(
        'attributes.Attribute',
        related_name='options',
        verbose_name=_("Attribute"),
        on_delete=models.CASCADE)

    name = models.CharField(_('Name'), max_length=255)

    def __str__(self):
        return self.name

    class Meta:
        unique_together = ('attr', 'name')
        verbose_name = _('Attribute option')
        verbose_name_plural = _('Attribute options')




from django.db.models import Manager

from attributes.querysets import AttributeValueQuerySet


class AttributeValueManager(Manager):

    def get_queryset(self):
        return AttributeValueQuerySet(self.model, using=self._db)

    def visible(self):
        return self.get_queryset().visible()




from django.utils.translation import gettext_lazy as _


ATTR_TYPE_TEXT = 1
ATTR_TYPE_INT = 2
ATTR_TYPE_BOOL = 3
ATTR_TYPE_SELECT = 5

ATTR_TYPES = (
    (ATTR_TYPE_TEXT, _("Text")),
    (ATTR_TYPE_INT, _("Integer")),
    (ATTR_TYPE_BOOL, _("True / False")),
    (ATTR_TYPE_SELECT, _("Options")),
)
